@@ -80,7 +80,7 @@ void file_search_add_file(file_search_data* data, tstr_t path, unsigned file_mod
 	}
 	if (IS_DASH_TSTR(path))
 	{
-		file_search_add_special_file(data, (FileIsRoot | FileIsStdin), NULL);
+		file_search_add_special_file(data, (file_mode | FileIsStdin), NULL);
 		return;
 	}
 
@@ -125,7 +125,7 @@ void file_search_add_file(file_search_data* data, tstr_t path, unsigned file_mod
 				filepath = make_wpath(parent, index + 1, d.cFileName);
 				if (!filepath)
 					continue;
-				res = file_init(&file, filepath, file_mode | FileInitUpdatePrintPathLastSlash | FileInitRunFstat);
+				res = file_init(&file, filepath, file_mode | FileInitUpdatePrintPathSlashes | FileInitRunFstat);
 				free(filepath);
 
 				/* convert file name */
@@ -459,7 +459,7 @@ static int dir_scan(file_t* start_dir, file_search_data* data)
 			filepath = make_tpath(dir_path, dirent_get_tname(de));
 			if (!filepath)
 				continue;
-			res  = file_init(&file, filepath, FileInitRunFstat | FileInitUpdatePrintPathLastSlash);
+			res  = file_init(&file, filepath, fstat_bit | FileInitUpdatePrintPathSlashes);
 			free(filepath);
 			if (res >= 0)
 			{
