@@ -1,13 +1,16 @@
 /* hash_check.c - verification of message digests of files */
 
 #include "hash_check.h"
+#include "calc_sums.h"
 #include "common_func.h"
 #include "hash_print.h"
 #include "output.h"
 #include "parse_cmdline.h"
+#include "rhash_main.h"
 #include "librhash/rhash.h"
 #include <assert.h>
 #include <ctype.h>  /* isspace */
+#include <errno.h>
 #include <string.h>
 
 /* message digest conversion macros and functions */
@@ -278,9 +281,10 @@ static unsigned char test_hash_string(char** ptr, char* end, int* p_len)
  */
 static unsigned bsd_hash_name_to_id(const char* name, unsigned length)
 {
-#define code2mask_size (18 * 2)
+#define code2mask_size (19 * 2)
 	static unsigned code2mask[code2mask_size] = {
 		FOURC2U('A', 'I', 'C', 'H'), RHASH_AICH,
+		FOURC2U('B', 'L', 'A', 'K'), (RHASH_BLAKE2S | RHASH_BLAKE2B),
 		FOURC2U('B', 'T', 'I', 'H'), RHASH_BTIH,
 		FOURC2U('C', 'R', 'C', '3'), (RHASH_CRC32 | RHASH_CRC32C),
 		FOURC2U('E', 'D', '2', 'K'), RHASH_ED2K,
@@ -441,7 +445,7 @@ static int hash_check_find_str(hc_search* search, const char* format)
 				hv.offset = (unsigned short)(end - hc->data);
 			} else {
 				hv.offset = (unsigned short)(begin - hc->data);
-				hv.format = test_hash_string(&begin, end, &len);
+				hv.format = test_hash_string(&begin, end, &len) & search->hash_type;
 			}
 			if (!hv.format) return 0;
 			if (*search_str == '\3') {
@@ -522,9 +526,11 @@ static int hash_check_find_str(hc_search* search, const char* format)
  *
  * @param line the line to parse
  * @param hashes structure to store parsed message digests, file path and file size
+ * @param expected_hash_mask hash mask of expected algorithms
+ * @param check_eol boolean flag meaning that '\n' at the end of the line is required
  * @return 1 on success, 0 if couldn't parse the line
  */
-int hash_check_parse_line(char* line, hash_check* hashes, int check_eol)
+int hash_check_parse_line(char* line, hash_check* hashes, unsigned expected_hash_mask, int check_eol)
 {
 	hc_search hs;
 	char* le = strchr(line, '\0'); /* set pointer to the end of line */
@@ -631,17 +637,22 @@ int hash_check_parse_line(char* line, hash_check* hashes, int check_eol)
 			while (hash_check_find_str(&hs, "$\6\2"));
 			if (hashes->hashes_num > 1)
 				reversed = 1;
-		} else if (hash_check_find_str(&hs, "\2\7")) {
-			if (hs.begin == hs.end) {
-				/* the line contains no file path, only a single hash */
-				single_hash = 1;
-			} else {
-				while (hash_check_find_str(&hs, "\2\6"));
-				/* drop an asterisk before filename if present */
-				if (*hs.begin == '*')
-					hs.begin++;
-			}
-		} else bad = 1;
+		} else {
+			hs.hash_type = FmtAll;
+			if (hash_check_find_str(&hs, "\2\7")) {
+				if (hs.begin == hs.end) {
+					/* the line contains no file path, only a single hash */
+					single_hash = 1;
+				} else {
+					if (hs.hc->hashes_num == 1 && hs.hc->hashes[0].format != FmtBase64)
+						hs.hash_type &= ~FmtBase64;
+					while (hash_check_find_str(&hs, "\2\6"));
+					/* drop an asterisk before filename if present */
+					if (*hs.begin == '*')
+						hs.begin++;
+				}
+			} else bad = 1;
+		}
 
 		if (hs.begin >= hs.end && !single_hash)
 			bad = 1;
@@ -696,8 +707,8 @@ int hash_check_parse_line(char* line, hash_check* hashes, int check_eol)
 				mask |= hash_check_mask_by_digest_size(BASE64_BIT_SIZE(hv->length) / 8);
 			}
 			assert(mask != 0);
-			if ((mask & opt.sum_flags) != 0)
-				mask &= opt.sum_flags;
+			if ((mask & expected_hash_mask) != 0)
+				mask &= expected_hash_mask;
 			hv->hash_id = mask;
 		}
 		hashes->hash_mask |= hv->hash_id;
@@ -856,3 +867,270 @@ int do_hash_sums_match(hash_check* hashes, struct rhash_context* ctx)
 		hashes->flags |= HC_WRONG_HASHES;
 	return !HC_FAILED(hashes->flags);
 }
+
+/**
+ * Verify message digests of the file.
+ * In a case of fail, the error will be logged.
+ *
+ * @param info structure file path to process
+ * @return 0 on success, 1 on message digests mismatch,
+ *     -1/-2 on input/output error
+ */
+static int verify_sums(struct file_info* info)
+{
+	timedelta_t timer;
+	int res = 0;
+
+	/* initialize percents output */
+	if (init_percents(info) < 0) {
+		log_error_file_t(&rhash_data.out_file);
+		return -2;
+	}
+	rsh_timer_start(&timer);
+
+	if (FILE_ISBAD(info->file) || calc_sums(info) < 0) {
+		return (finish_percents(info, -1) < 0 ? -2 : -1);
+	}
+	info->time = rsh_timer_stop(&timer);
+
+	if (rhash_data.stop_flags) {
+		report_interrupted();
+		return 0;
+	}
+
+	if ((opt.flags & OPT_EMBED_CRC) &&
+			find_embedded_crc32(info->file, &info->hc.embedded_crc32)) {
+		info->hc.flags |= HC_HAS_EMBCRC32;
+		assert(info->hc.hash_mask & RHASH_CRC32);
+	}
+
+	if (!do_hash_sums_match(&info->hc, info->rctx))
+		res = 1;
+
+	if (finish_percents(info, res) < 0)
+		res = -2;
+
+	if ((opt.flags & OPT_SPEED) && info->sums_flags) {
+		print_file_time_stats(info);
+	}
+	return res;
+}
+
+/**
+ * Verify the file against the CRC32 checksum embedded into its filename.
+ *
+ * @param file the file to verify
+ * @return 0 on success, -1 on input error, -2 on results output error
+ */
+static int check_embedded_crc32(file_t* file)
+{
+	int res = 0;
+	unsigned crc32;
+	struct file_info info;
+	if (find_embedded_crc32(file, &crc32)) {
+		/* initialize file_info structure */
+		memset(&info, 0, sizeof(info));
+		info.file = file;
+		info.sums_flags = info.hc.hash_mask = RHASH_CRC32;
+		info.hc.flags = HC_HAS_EMBCRC32;
+		info.hc.embedded_crc32 = crc32;
+
+		res = verify_sums(&info);
+		if (res >= -1 && fflush(rhash_data.out) < 0) {
+			log_error_file_t(&rhash_data.out_file);
+			res = -2;
+		} else if (!rhash_data.stop_flags) {
+			if (res >= 0)
+				rhash_data.ok++;
+			else if (res == -1 && errno == ENOENT)
+				rhash_data.miss++;
+			rhash_data.processed++;
+		}
+	} else {
+		/* TRANSLATORS: sample filename with embedded CRC32: file_[A1B2C3D4].mkv */
+		log_warning_msg_file_t(_("file name doesn't contain a CRC32: %s\n"), file);
+		return -1;
+	}
+	return res;
+}
+
+/*
+ * Detect hash mask by the file extension.
+ *
+ * @param file the file which extension will be checked
+ * @return hash_id on success, 0 on fail
+ */
+static unsigned hash_mask_by_file_ext(file_t* file)
+{
+	const char* basename = file_get_print_path(file, FPathUtf8 | FPathBaseName);
+	if (basename) {
+		const char* ext = strrchr(basename, '.');
+		if (ext && *(++ext) != '\0') {
+			size_t length;
+			char buffer[20];
+			unsigned hash_id;
+			for (length = 0; '-' <= ext[length] && ext[length] <= 'z'; length++) {
+				if (length >= 20)
+					return 0; /* limit hash name length */
+				buffer[length] = toupper(ext[length]);
+			}
+			if (ext[length] == '\0') {
+				buffer[length] = '\0';
+				hash_id = bsd_hash_name_to_id(buffer, length);
+				if (hash_id != 0)
+					return hash_id;
+			}
+		}
+	}
+	return 0;
+}
+
+/**
+ * Verify message digests in a hash file.
+ * Lines beginning with ';' and '#' are ignored.
+ * In a case of fail, the error will be logged.
+ *
+ * @param file the file containing message digests to verify.
+ * @param chdir true if function should emulate chdir to directory of filepath before checking it.
+ * @return 0 on success, -1 on input error, -2 on results output error
+ */
+int check_hash_file(file_t* file, int chdir)
+{
+	FILE* fd;
+	file_t parent_dir;
+	file_t* p_parent_dir = 0;
+	char buf[4096];
+	timedelta_t timer;
+	struct file_info info;
+	int res = 0;
+	int line_number = 0;
+	unsigned init_flags = 0;
+	unsigned expected_hash_mask = opt.sum_flags;
+	double time;
+
+	/* process --check-embedded option */
+	if (opt.mode & MODE_CHECK_EMBEDDED)
+		return check_embedded_crc32(file);
+
+	/* initialize statistics */
+	rhash_data.processed = rhash_data.ok = rhash_data.miss = 0;
+	rhash_data.total_size = 0;
+
+	/* open file / prepare file descriptor */
+	if (FILE_ISSTDIN(file)) {
+		fd = stdin;
+	} else if ( !(fd = file_fopen(file, FOpenRead | FOpenBin) )) {
+		log_error_file_t(file);
+		return -1;
+	}
+
+	if (print_verifying_header(file) < 0) {
+		log_error_file_t(&rhash_data.out_file);
+		if (fd != stdin)
+			fclose(fd);
+		return -2;
+	}
+	rsh_timer_start(&timer);
+	memset(&parent_dir, 0, sizeof(parent_dir));
+
+	if (chdir) {
+		/* extract the parent directory */
+		file_modify_path(&parent_dir, file, NULL, FModifyGetParentDir);
+		p_parent_dir = &parent_dir;
+	}
+	if (!expected_hash_mask && !(opt.flags & OPT_NO_DETECT_BY_EXT))
+		expected_hash_mask = hash_mask_by_file_ext(file);
+
+	/* read hash file line by line */
+	for (line_number = 0; fgets(buf, sizeof(buf), fd); line_number++) {
+		char* line = buf;
+		file_t file_to_check;
+
+		/* skip unicode BOM */
+		if (STARTS_WITH_UTF8_BOM(buf)) {
+			line += 3;
+			if (line_number == 0)
+				init_flags = FileInitUtf8PrintPath; /* hash file is in UTF8 */
+		}
+
+		if (*line == 0)
+			continue; /* skip empty lines */
+
+		if (is_binary_string(line)) {
+			log_error(_("file is binary: %s:%d\n"), file_get_print_path(file, FPathPrimaryEncoding | FPathNotNull), line_number + 1);
+			if (fd != stdin)
+				fclose(fd);
+			file_cleanup(&parent_dir);
+			return -1;
+		}
+
+		/* skip comments and empty lines */
+		if (IS_COMMENT(*line) || *line == '\r' || *line == '\n')
+			continue;
+
+		memset(&info, 0, sizeof(info));
+
+		if (!hash_check_parse_line(line, &info.hc, expected_hash_mask, !feof(fd)))
+			continue;
+		if (info.hc.hash_mask == 0)
+			continue;
+
+		/* check if the hash file contains a message digest without a filename */
+		if (info.hc.file_path != NULL) {
+			int is_absolute = IS_PATH_SEPARATOR(info.hc.file_path[0]);
+			IF_WINDOWS(is_absolute = is_absolute || (info.hc.file_path[0] && info.hc.file_path[1] == ':'));
+			file_init_by_print_path(&file_to_check, (is_absolute ? NULL : p_parent_dir), info.hc.file_path, init_flags);
+		} else {
+			if (file_modify_path(&file_to_check, file, NULL, FModifyRemoveExtension) < 0) {
+				/* note: trailing whitespaces were removed from line by hash_check_parse_line() */
+				log_error(_("%s: can't parse line \"%s\"\n"), file_get_print_path(file, FPathPrimaryEncoding | FPathNotNull), line);
+				continue;
+			}
+		}
+
+		info.file = &file_to_check;
+		info.sums_flags = info.hc.hash_mask;
+		file_stat(&file_to_check, 0);
+
+		/* verify message digests of the file */
+		res = verify_sums(&info);
+
+		if (res >= -1 && fflush(rhash_data.out) < 0) {
+			log_error_file_t(&rhash_data.out_file);
+			res = -2;
+		}
+		file_cleanup(&file_to_check);
+
+		if (rhash_data.stop_flags || res < -1) {
+			break; /* stop on fatal error */
+		}
+
+		/* update statistics */
+		if (res == 0)
+			rhash_data.ok++;
+		else if (res == -1 && errno == ENOENT)
+			rhash_data.miss++;
+		rhash_data.processed++;
+	}
+	file_cleanup(&parent_dir);
+	time = rsh_timer_stop(&timer);
+
+	if (res >= -1 && (print_verifying_footer() < 0 || print_check_stats() < 0)) {
+		log_error_file_t(&rhash_data.out_file);
+		res = -2;
+	}
+
+	if (rhash_data.processed != rhash_data.ok)
+		rhash_data.non_fatal_error = 1;
+
+	if ((opt.flags & OPT_SPEED) && rhash_data.processed > 1)
+		print_time_stats(time, rhash_data.total_size, 1);
+
+	rhash_data.processed = 0;
+	/* check for input errors */
+	if (res >= 0 && ferror(fd))
+		res = -1;
+	if (fd != stdin)
+		fclose(fd);
+	return res;
+}
